﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Threading;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Xaml.Shapes;

// Pour en savoir plus sur le modèle d'élément Page vierge, consultez la page http://go.microsoft.com/fwlink/?LinkId=234238

namespace GenTrix
{
    /// <summary>
    /// Une page vide peut être utilisée seule ou constituer une page de destination au sein d'un frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public DispatcherTimer timerLoop;

        public Shape Rect;
        public Dictionary<int, List<int>> Matrix = new Dictionary<int, List<int>>();
        public Random NextPieces = new Random();

        public int PiecesStep { get; set; }
        public int LeftMoove { get; set; }
        public int RightMoove { get; set; }
        public int DownMoove { get; set; }
        public int DirectionMove { get; set; }

        public enum Direction
        {
            Left = 0,
            Right = 1,
            Up = 2,
            Down = 3
        }

        public MainPage()
        {
            this.InitializeComponent();
            NextStep();
            Wait();
            CoreWindow.GetForCurrentThread().KeyUp += MainPageCore_KeyDown;
            CoreWindow.GetForCurrentThread().KeyDown += MainPageCore_KeyDown;
            CanvasMain.KeyDown +=MainPage_KeyDown;
            KeyDown += MainPage_KeyDown;
        }

        void MainPage_KeyDown(object sender, KeyRoutedEventArgs args)
        {
            if (args.Key == Windows.System.VirtualKey.Left)
            {
                LeftMoove += 5;
            }
            else if (args.Key == Windows.System.VirtualKey.Right)
            {
                RightMoove += 5;
            }
            else if (args.Key == Windows.System.VirtualKey.Down)
            {
                DownMoove += 5;
            };
        }

        void MainPageCore_KeyDown(CoreWindow e, KeyEventArgs args)
        {
            if (args.VirtualKey == Windows.System.VirtualKey.Left)
            {
                LeftMoove += 5;
            }
            else if (args.VirtualKey == Windows.System.VirtualKey.Right)
            {
                RightMoove += 5;
            }
            else if (args.VirtualKey == Windows.System.VirtualKey.Down)
            {
                DownMoove += 5;
            }
        }

        public enum Step
        {
            Square = 1,
            Line = 2,
            Triangle = 3,
            Trigone = 4
        }

        public void NextStep()
        {
            PiecesStep = NextPieces.Next(1, 5);
            LeftMoove = 0;
            RightMoove = 0;
            DownMoove = 0;
        }

        private void CreateShape() 
        {
            var random = new Random();
            switch (PiecesStep)
            {
                case 1:
                    Rect = CreateCarre();
                    break;
                case 2:
                    Rect = CreateTrigone();
                    break;
                case 3:
                    Rect = CreatePolygon();
                    break;
                case 4:
                    Rect = CreateRectangle();
                    break;
            }
        }

        public void Wait()
        {
            CreateShape();
            timerLoop = new DispatcherTimer();
            timerLoop.Interval = TimeSpan.FromMilliseconds(50);
            timerLoop.Tick += TimerLoopTick;
            timerLoop.Start();
        }

        public int PositionX { get; set; }
        public int PositionY { get; set; }
        public Random random = new Random();

        public Shape CreateRectangle()
        {
            Rectangle rect = new Rectangle();
            SolidColorBrush blueBrush = new SolidColorBrush(Windows.UI.Colors.Maroon);
            rect.Fill = blueBrush;
            rect.StrokeThickness = 2;
            rect.Stroke = new SolidColorBrush(Windows.UI.Colors.Black);
            rect.Width = 25;
            rect.Height = 100;
            CanvasMain.Children.Add(rect);
            Canvas.SetLeft(rect, PositionX + RightMoove - LeftMoove);
            Canvas.SetTop(rect, PositionY + DownMoove);
            return rect;
        }

        public Shape CreateCarre()
        {
            Rectangle rect = new Rectangle();
            SolidColorBrush blueBrush = new SolidColorBrush(Windows.UI.Colors.DeepSkyBlue);
            rect.Fill = blueBrush;
            rect.StrokeThickness = 2;
            rect.Stroke = new SolidColorBrush(Windows.UI.Colors.Black);
            rect.Width = 50;
            rect.Height = 50;
            CanvasMain.Children.Add(rect);
            Canvas.SetLeft(rect, PositionX +  RightMoove - LeftMoove);
            Canvas.SetTop(rect, PositionY + DownMoove);
            return rect;
        }

        public Shape CreatePolygon()
        {
            SolidColorBrush blueBrush = new SolidColorBrush(Windows.UI.Colors.YellowGreen);
            Polygon polygon = new Polygon();
            polygon.StrokeThickness = 2;
            polygon.Stroke = new SolidColorBrush(Windows.UI.Colors.Black);
            polygon.Points.Add(new Point(0 + RightMoove - LeftMoove, 25 + DownMoove));
            polygon.Points.Add(new Point(100 + RightMoove - LeftMoove, 25 + DownMoove));
            polygon.Points.Add(new Point(100 + RightMoove - LeftMoove, 50 + DownMoove));
            polygon.Points.Add(new Point(0 + RightMoove - LeftMoove, 50 + DownMoove));
            polygon.Points.Add(new Point(0 + RightMoove - LeftMoove, 50 + DownMoove));
            polygon.Points.Add(new Point(25 + RightMoove - LeftMoove, 50 + DownMoove));
            polygon.Points.Add(new Point(25 + RightMoove - LeftMoove, 75 + DownMoove));
            polygon.Points.Add(new Point(0 + RightMoove - LeftMoove, 75 + DownMoove));
            polygon.Fill = blueBrush;
            CanvasMain.Children.Add(polygon);
            Canvas.SetLeft(polygon, PositionX);
            Canvas.SetTop(polygon, PositionY);
            return polygon;
        }

        public Shape CreateTrigone()
        {
            SolidColorBrush blueBrush = new SolidColorBrush(Windows.UI.Colors.OrangeRed);
            Polygon polygon = new Polygon();
            polygon.StrokeThickness = 2;
            polygon.Stroke = new SolidColorBrush(Windows.UI.Colors.Black);
            polygon.Points.Add(new Point(0 + RightMoove - LeftMoove, 25 + DownMoove));
            polygon.Points.Add(new Point(75 + RightMoove - LeftMoove, 25 + DownMoove));
            polygon.Points.Add(new Point(75 + RightMoove - LeftMoove, 50 + DownMoove));
            polygon.Points.Add(new Point(50 + RightMoove - LeftMoove, 50 + DownMoove));
            polygon.Points.Add(new Point(50 + RightMoove - LeftMoove, 75 + DownMoove));
            polygon.Points.Add(new Point(25 + RightMoove - LeftMoove, 75 + DownMoove));
            polygon.Points.Add(new Point(25 + RightMoove - LeftMoove, 50 + DownMoove));
            polygon.Points.Add(new Point(0 + RightMoove - LeftMoove, 50 + DownMoove));

            //polygon.Points.Add(new Point(25, 0));
            polygon.Fill = blueBrush;
            CanvasMain.Children.Add(polygon);
            Canvas.SetLeft(polygon, PositionX);
            Canvas.SetTop(polygon, PositionY);
            return polygon;
        }

        public int DecalageCarre() { return 50; }
        public int DecalageLine() { return 100; }
        public int DecalageTrigone() { return 50; }
        public int DecalageRectangle() { return 50; }

        public void TimerLoopTick(object sender, object e)
        {
            var decalageX = PositionY;
            switch (PiecesStep)
            {
                case 1:
                    //carre
                    decalageX = PositionY + DecalageCarre();
                    break;
                case 2:
                    //trigone
                    decalageX = PositionY + DecalageTrigone();
                    break;
                case 3:
                    //polygone
                    decalageX = PositionY + DecalageRectangle();
                    break;
                case 4:
                    //rectangle
                    decalageX = PositionY + DecalageLine();
                    break;
            }

            if (Matrix.ContainsKey(PositionX + RightMoove - LeftMoove) && Matrix[PositionX + RightMoove - LeftMoove].Contains(decalageX))
            {
                StopAnimation();
                return;
            }
       
            CanvasMain.Children.Remove(Rect);
            PositionY = PositionY + 5;
            CreateShape();
            if (PositionY + DownMoove >= 690)
                StopAnimation();
        }

        public void StrategyCarre() 
        {
            PositionX = random.Next(0, 750);
            if (PositionX > 0 && PositionX <= 50)
            {
                PositionX = 0;
            }
            else if (PositionX > 50 && PositionX <= 100)
            {
                PositionX = 50;
            }
            else if (PositionX > 100 && PositionX <= 150)
            {
                PositionX = 100;
            }
            else if (PositionX > 150 && PositionX <= 200)
            {
                PositionX = 150;
            }
            else if (PositionX > 200 && PositionX <= 250)
            {
                PositionX = 200;
            }
            else if (PositionX > 250 && PositionX <= 300)
            {
                PositionX = 250;
            }
            else if (PositionX > 300 && PositionX <= 350)
            {
                PositionX = 350;
            }
            else if (PositionX > 350 && PositionX <= 400)
            {
                PositionX = 400;
            }
            else if (PositionX > 400 && PositionX <= 450)
            {
                PositionX = 450;
            }
            else if (PositionX > 450 && PositionX <= 500)
            {
                PositionX = 500;
            }
            else if (PositionX > 500 && PositionX <= 550)
            {
                PositionX = 550;
            }
            else if (PositionX > 550 && PositionX <= 600)
            {
                PositionX = 600;
            }
            else if (PositionX > 600 && PositionX <= 650)
            {
                PositionX = 650;
            }
            else if (PositionX > 650 && PositionX <= 700)
            {
                PositionX = 700;
            }
            else if (PositionX > 700 && PositionX <= 750)
            {
                PositionX = 750;
            }
        }

        public void StrategyLine()
        {
            PositionX = random.Next(0, 750);
            for (var i = 0; i < 2; i++)
            {
                if (PositionX > 0 + (400 * i) && PositionX <= 25 + (400 * i))
                {
                    PositionX = 0 + (400 * i);
                }
                else if (PositionX > 25 + (400 * i) && PositionX <= 50 + (400 * i))
                {
                    PositionX = 25 + (400 * i);
                }
                else if (PositionX > 50 + (400 * i) && PositionX <= 75 + (400 * i))
                {
                    PositionX = 50 + (400 * i);
                }
                else if (PositionX > 75 + (400 * i) && PositionX <= 100 + (400 * i))
                {
                    PositionX = 75 + (400 * i);
                }
                else if (PositionX > 100 + (400 * i) && PositionX <= 125 + (400 * i))
                {
                    PositionX = 100 + (400 * i);
                }
                else if (PositionX > 125 + (400 * i) && PositionX <= 150 + (400 * i))
                {
                    PositionX = 125 + (400 * i);
                }
                else if (PositionX > 150 + (400 * i) && PositionX <= 175 + (400 * i))
                {
                    PositionX = 150 + (400 * i);
                }
                else if (PositionX > 175 + (400 * i) && PositionX <= 200 + (400 * i))
                {
                    PositionX = 175 + (400 * i);
                }
                else if (PositionX > 200 + (400 * i) && PositionX <= 225 + (400 * i))
                {
                    PositionX = 200 + (400 * i);
                }
                else if (PositionX > 225 + (400 * i) && PositionX <= 250 + (400 * i))
                {
                    PositionX = 225 + (400 * i);
                }
                else if (PositionX > 250 + (400 * i) && PositionX <= 275 + (400 * i))
                {
                    PositionX = 250 + (400 * i);
                }
                else if (PositionX > 275 + (400 * i) && PositionX <= 300 + (400 * i))
                {
                    PositionX = 275 + (400 * i);
                }
                else if (PositionX > 300 + (400 * i) && PositionX <= 325 + (400 * i))
                {
                    PositionX = 300 + (400 * i);
                }
                else if (PositionX > 325 + (400 * i) && PositionX <= 350 + (400 * i))
                {
                    PositionX = 325 + (400 * i);
                }
                else if (PositionX > 350 + (400 * i) && PositionX <= 400 + (400 * i))
                {
                    PositionX = 350 + (400 * i);
                }
            }
        }

        public void StartAnimation()
        {
            switch (PiecesStep)
            { 
                case 1:
                    StrategyCarre();
                    break;
                case 2:
                    StrategyLine();
                    break;
            }
            
            timerLoop.Start();
        }
        public void StopAnimation()
        {
            timerLoop.Stop();
            if (Matrix.ContainsKey(PositionX))
            {
                if (!Matrix[PositionX].Contains(PositionY))
                {
                    Matrix[PositionX].Add(PositionY);
                }
            }
            else 
            {
                Matrix.Add(PositionX, new List<int> { PositionY - 15});
            }
            CreateShape();
            PositionY = 0;
            StartAnimation();
            NextStep();
        }
    }
}
